use strict;
use warnings;
package CouchDB::Utils;

# ABSTRACT: couchdb command line utils

our $VERSION = '0.2';

1;

__END__

=pod

=head1 NAME

CouchDB::Utils - couchdb command line utils

=head1 VERSION

version 0.2

=head1 AUTHOR

Maroun NAJM <mnajm@cinemoz.com>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2012 by Cinemoz Inc.

This is free software, licensed under:

  The (three-clause) BSD License

=cut
